// SPDX-FileCopyrightText: 2023 g10 code GmbH
// SPDX-Contributor: Carl Schwan <carl.schwan@gnupg.com>
// SPDX-License-Identifier: GPL-2.0-or-later

#pragma once

#include <QHttpServerRequest>
#include <QHttpServerResponse>

class EmailController
{
public:
    static QHttpServerResponse viewEmailAction(const QHttpServerRequest &request);
    static QHttpServerResponse infoEmailAction(const QHttpServerRequest &request);
    static QHttpServerResponse newEmailAction(const QHttpServerRequest &request);
    static QHttpServerResponse forwardEmailAction(const QHttpServerRequest &request);
    static QHttpServerResponse replyEmailAction(const QHttpServerRequest &request);
    static QHttpServerResponse draftAction(QString draftId, const QHttpServerRequest &request);
};