const messages = {"fr-FR": {"Loading...": "Chargement...", "This mail is encrypted and signed.": "Ce email est encrypt\u00e9 et sign\u00e9 num\u00e9riquement", "This mail is encrypted.": "Ce email est encrypt\u00e9"}, "de-DE": {"Loading...": "Laden...", "This mail is encrypted and signed.": "Diese E-Mail ist verschl\u00fcsselt und signiert", "This mail is encrypted.": "Diese E-Mail ist verschl\u00fcsselt", "This mail is signed": "Diese E-Mail ist signiert", "This mail is not encrypted nor signed.": "Diese E-Mail ist nicht verschl\u00fcsselt und signiert", "Decrypt": "Entschl\u00fcsseln", "View email": "E-Mail anzeigen"}}