<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd" >
<gui version="1" name="composer" translationDomain="gpgol-server">
 <MenuBar>
  <Menu noMerge="1" name="file" >
   <text>&amp;Message</text>
   <Action name="save_as_file" />
   <Action name="save_in_drafts" />
   <Separator/>
   <Action name="file_print_preview"/>
   <Action name="file_print" />
   <Separator/>
   <Action name="file_close" />
  </Menu>
  <Menu name="edit" >
   <text>&amp;Edit</text>
   <Action name="paste_quoted" append="edit_paste_merge"/>
   <Action name="paste_att" append="edit_paste_merge"/>
   <Action name="paste_without_formatting" append="edit_paste_merge"/>
   <Action name="clean_spaces" />
   <Separator/>
   <Action name="tools_quote"/>
   <Action name="tools_unquote"/>
  </Menu>
  <Menu name="insert" >
   <text>Insert</text>
   <Action name="append_signature" />
   <Action name="prepend_signature" />
   <Action name="insert_signature_at_cursor_position" />
   <Separator/>
   <Action name="insert_file" />
   <Action name="insert_file_recent" />
  </Menu>
  <Menu noMerge="1" name="options" >
   <text>&amp;Options</text>
   <Action name="sign_message" />
   <Action name="encrypt_message" />
   <Separator/>
   <Action name="options_select_crypto" />
   <Action name="html"/>
   <Action name="wordwrap" />
   <Action name="options_auto_spellchecking" />
  </Menu>
  <Menu name="attach" >
   <text>&amp;Attach</text>
   <Action name="attach_public_key" />
   <Action name="attach_my_public_key" />
   <Separator/>
   <Action name="attach_menu" />
   <Action name="remove" />
   <Action name="attach_save" />
   <Action name="attach_properties" />
   <!--<Separator/>
   <Action name="shared_link" />-->
  </Menu>
  <Menu name="tools">
   <Action name="spellcheck" />
  </Menu>
  <Menu name="settings" >
   <text>&amp;Settings</text>
   <Action name="setup_spellchecker" append="save_merge"/>
   <Action name="options_show_menubar" group="settings_configure" />
  </Menu>
 </MenuBar>
 <ToolBar noMerge="1" name="mainToolBar" fullWidth="true" >
  <text>Main Toolbar</text>
  <Action name="mail_send" />
  <Separator/>
  <Action name="html" />
  <Separator/>
  <Action name="spellcheck" />
  <Separator/>
  <Action name="attach_menu" />
  <Separator/>
  <Action name="encrypt_message" />
  <Separator/>
  <ActionList name="kmaileditor_toolbar_actions"/>
  <Spacer/>
  <Action name="hamburger_menu"/>
 </ToolBar>
 <ToolBar noMerge="1" newline="true" hidden="true" enable="0" name="htmlToolBar" iconText="icononly" >
  <text>HTML Toolbar</text>
  <Action name="format_heading_level" />
  <Action name="format_list_style" />
  <Action name="format_font_family" />
  <Action name="format_font_size" />
  <Action name="format_list_checkbox" />
  <Separator/>
  <Action name="format_reset" />
  <Separator/>
  <Action name="format_painter" />
  <Action name="format_text_bold" />
  <Action name="format_text_italic" />
  <Action name="format_text_underline" />
  <Action name="format_text_strikeout" />
  <Action name="format_text_foreground_color" />
  <Action name="format_text_background_color" />
  <Separator/>
  <Action name="format_align_left" />
  <Action name="format_align_center" />
  <Action name="format_align_right" />
  <Action name="format_align_justify" />
  <Separator/>
  <Action name="format_list_indent_more"/>
  <Action name="format_list_indent_less"/>
  <Separator/>
  <Action name="manage_link" />
  <Separator/>
  <Action name="insert_horizontal_rule" />
  <Separator/>
  <Action name="add_image" />
  <Separator/>
  <Action name="insert_html" />
  <Separator/>
  <Action name="insert_table" />
 </ToolBar>
 <ToolBar noMerge="1" newline="true" hidden="true" enable="0" name="directionToolBar" iconText="icononly" >
  <text>Text Direction Toolbar</text>
  <Action name="direction_ltr" />
  <Action name="direction_rtl" />
 </ToolBar>
</gui>
